<?php

/**
 * @file
 * Contains \Drupal\search_api_hashtag\Plugin\search_api\processor\HtmlFilter.
 */

namespace Drupal\search_api_hashtag\Plugin\search_api\processor;

use Drupal\Core\Form\FormStateInterface;
use Drupal\search_api\Datasource\DatasourceInterface;
use Drupal\search_api\Processor\FieldsProcessorPluginBase;
use Drupal\search_api\Property\BasicProperty;

/**
 * @SearchApiProcessor(
 *   id = "hashtag",
 *   label = @Translation("Hashtags"),
 *   description = @Translation("HashtagProcessor items extracted from text."),
 *   stages = {
 *     "preprocess_index" = 0
 *   }
 * )
 */
class HashtagProcessor extends FieldsProcessorPluginBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return array(
      'fields' => array(),
      'strtolower' => 0
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    unset($form['fields']['#options']['search_api_hashtag']);

    $form['lowercase'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('Convert all indexed hashtags values to lowercase'),
      '#default_value' => $this->configuration['lowercase'],
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::validateConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function alterPropertyDefinitions(array &$properties, DatasourceInterface $datasource = NULL) {
    if ($datasource) {
      return;
    }

    $definition = array(
      'label' => $this->t('Hashtag'),
      'description' => $this->t('Hashtag items extracted from text'),
      'type' => 'string',
    );

    $properties['search_api_hashtag'] = BasicProperty::createFromDefinition($definition)->setLocked();
  }

  /**
   * {@inheritdoc}
   */
  public function preprocessIndexItems(array &$items) {
    // Annoyingly, this doc comment is needed for PHPStorm. See
    // http://youtrack.jetbrains.com/issue/WI-23586
    /** @var \Drupal\search_api\Item\ItemInterface $item */
    foreach ($items as $item) {
      if (!($field = $item->getField('search_api_hashtag'))) {
        continue;
      }
      $config = $this->getConfiguration();
      $hashtag = array();
      foreach ($config['fields'] as $filter_field) {
        preg_match_all('/#(\w+)/', $item->getField($filter_field), $matches);
        $field_hashtags = $config['lowercase'] ? array_map('strtolower', $matches[1]) : $matches[1];
        $hashtag = array_merge($hashtag, $field_hashtags);
      }

      $hashtag = array_values(array_unique($hashtag));

      foreach ($hashtag as $value) {
        $field->addValue($value);
      }
    }
  }

}
